# Access of PortfolioData columns
INTEREST <- 2 # interest column (risk free rate)
BOND <- 3 # Cash from zero bonds
DEPOSIT <- 4 # Deposit by policyholders with the 
             # corresponding maturity.
tiny <- 1E-6
# Our three scenarios
BE <- 1
UP <- 2
DOWN <- 3

# Maturity t, rel. change sup(t), re. change sdown(t)
# Durations shorter than 1 year have the same relative
# changes as duration 1
Shocks <- rbind(c(1, 70/100, -75/100),
                c(2, 70/100, -65/100),
                c(3, 64/100, -56/100),
                c(4, 59/100, -50/100),
                c(5, 55/100, -46/100),
                c(6, 52/100, -42/100),
                c(7, 49/100, -39/100),
                c(8, 47/100, -36/100),
                c(9, 44/100, -33/100),
                c(10, 42/100, -31/100),
                c(11, 39/100, -30/100),
                c(12, 37/100, -29/100),
                c(13, 35/100, -28/100),
                c(14, 34/100, -28/100),
                c(15, 33/100, -27/100),
                c(16, 31/100, -28/100),
                c(17, 30/100, -28/100),
                c(18, 29/100, -28/100),
                c(19, 27/100, -29/100),
                c(20, 26/100, -29/100),
                c(21, 26/100, -29/100),
                c(22, 26/100, -30/100),
                c(23, 26/100, -30/100),
                c(24, 26/100, -30/100),
                c(25, 26/100, -30/100),
                c(30, 25/100, -30/100))
Bonds <- rbind(c(1,1000),
               c(2,500),
               c(3,200),
               c(4,100))
PortfolioData <- rbind (c(1,   0.5/100, 1200, 500),
                        c(2,   1.0/100,  500, 450),
                        c(3,   1.2/100,  300, 300),
                        c(4,   1.5/100,  100, 250),
                        c(5,   2.0/100,    0, 200),
                        c(6,   2.2/100,    0, 150),
                        c(7,   2.3/100,    0, 100),
                        c(8,   2.3/100,    0,  50))
Projection <- 1:length(PortfolioData[,1])

# Initialization of vectors and matrices
discount <- matrix(NA,3,length(Projection))
Assets <- c()
Claims <- matrix(NA,3,length(Projection))
Liabs <- rep(NA,3)
NAV <- rep(NA,3)
D_NAV <- rep(NA,3)
ADJ_Liabs <- rep(NA,3)
ADJ_NAV <- rep(NA,3)
ADJ_D_NAV <- rep(NA,3)

# risk free rate for the three scenarios: BE, UP, DOWN
RF <- rbind(PortfolioData[,INTEREST],
           PortfolioData[,INTEREST]
            *(1+Shocks[Projection,UP]),
            pmax(PortfolioData[,INTEREST]
                 +pmin(PortfolioData[,INTEREST]
                       *Shocks[Projection,DOWN],
                       -0.01),0))

# The policyholders are credited 90% of risk free rate
CreditFactor <- 0.9 
for (i in c(BE,UP,DOWN)){
  Claims[i,] <- (cumprod(1 +CreditFactor*RF[i,])
                        *PortfolioData[,DEPOSIT])
}
for (i in c(BE,UP,DOWN)){
  discount[i,] <- 1/cumprod(1+RF[i,])
  Assets[i] <- sum(discount[i,]*PortfolioData[,BOND])
  # Without risk mitigation we use the best estimate
  # claims in all scenarios
  Liabs[i] <- sum(discount[i,]*Claims[BE,])
  NAV[i] <- Assets[i]-Liabs[i]
  ADJ_Liabs[i] <- sum(discount[i,]*Claims[i,])
  ADJ_NAV[i] <- Assets[i]-ADJ_Liabs[i]
}
for (i in c(UP,DOWN)){
  D_NAV[i] <- max(0,-(NAV[i]-NAV[BE]))
  ADJ_D_NAV[i] <- max(0,-(ADJ_NAV[i]-ADJ_NAV[BE]))
}

ADJ_SCR <- max(ADJ_D_NAV[UP],ADJ_D_NAV[DOWN],0)

#tiny rather 0 to avert rounding problems
if (ADJ_SCR < tiny){  
  SCR <- 0
}else if (abs(ADJ_SCR-ADJ_D_NAV[UP]) < tiny){
  SCR <- D_NAV[UP]
}else{
  SCR <- D_NAV[DOWN]
}

Results <- data.frame(row.names = c("BE","UP","DOWN"),
                      Assets,
                      Liabs,
                      NAV,
                      D_NAV,
                      ADJ_Liabs,
                      ADJ_NAV,
                      ADJ_D_NAV
                      )
names(Results) = (c("Assets",
                    "Liabs",
                    "NAV",
                    "D_NAV",
                    "ADJ_Liabs",
                    "ADJ_NAV",
                    "ADJ_D_NAV"))
print(Results)
SCR_Result <- data.frame(SCR,ADJ_SCR)
names(SCR_Result) <- c("SCR","ADJ_SCR")
print(SCR_Result)

